TaxesMod = {}
TaxesMod.modDirectory = g_currentModDirectory
TaxesMod.lastPeriodChecked = 0

TaxesMod.taxBrackets = {
    {limit = 2900, rate = 0.10}, {limit = 11787.5, rate = 0.12}, {limit = 25131.25, rate = 0.22},
    {limit = 47987.5, rate = 0.24}, {limit = 60931.25, rate = 0.32}, {limit = 152337.5, rate = 0.35},
    {limit = math.huge, rate = 0.37}
}

function TaxesMod:calculateTax(balance)
    if balance <= 0 then return 0 end
    local tax, previousLimit = 0, 0
    for _, bracket in ipairs(self.taxBrackets) do
        if balance > previousLimit then
            local taxable = math.min(balance, bracket.limit) - previousLimit
            if taxable > 0 then tax = tax + taxable * bracket.rate end
            previousLimit = bracket.limit
        else break end
    end
    return math.floor(tax)
end

function TaxesMod:processQuarterlyTax()
    if not g_farmManager or not g_farmManager.farms then
        print("TaxesMod: Farm manager or farms table not available")
        return
    end

    local taxes = {}
    for farmId, farm in pairs(g_farmManager.farms) do
        local farmIdNum = tonumber(farmId)
        if not farmIdNum then
            print("TaxesMod: farmId '" .. tostring(farmId) .. "' invalid, type: " .. type(farmId))
        elseif farmIdNum == FarmlandManager.NO_OWNER_FARM_ID or farmIdNum <= 0 or farmIdNum > 8 then
            print("TaxesMod: Skipping farmId " .. farmIdNum .. " (invalid or exceeds max 8)")
        else
            local farmObject = g_farmManager:getFarmById(farmIdNum)
            if farmObject then
                local balance = farmObject.money or 0
                local tax = self:calculateTax(balance)
                if tax > 0 then
                    farmObject:changeBalance(-tax, MoneyType.OTHER)
                    taxes[farmIdNum] = {balance = balance, amount = -tax}
                    print("TaxesMod: Farm " .. farmIdNum .. " paid $" .. tax .. " (balance: $" .. balance .. ")")
                end
            end
        end
    end

    if g_server and next(taxes) then
        self:syncTaxes(taxes)
    end
end

function TaxesMod:syncTaxes(taxes)
    local event = TaxSyncEvent.new(taxes)
    for _, connection in pairs(g_server.clientConnections) do
        if connection ~= g_server.serverConnection then
            connection:sendEvent(event)
            print("TaxesMod: Sent tax sync to client: " .. tostring(connection))
        end
    end
end

function TaxesMod:periodChanged(farm)
    local period = g_currentMission.environment.currentPeriod
    if period and period ~= self.lastPeriodChecked then
        self.lastPeriodChecked = period
        print("TaxesMod: Period changed to " .. period)
        if period % 3 == 0 then
            print("TaxesMod: Quarterly tax period detected")
            if g_server then self:processQuarterlyTax() end
        end
    end
end

function TaxesMod:loadMap()
    Farm.periodChanged = Utils.appendedFunction(Farm.periodChanged, function(farm)
        TaxesMod:periodChanged(farm)
    end)
    print("TaxesMod: Hooked Farm.periodChanged")
end

TaxSyncEvent = {}
TaxSyncEvent_mt = Class(TaxSyncEvent, Event)
InitEventClass(TaxSyncEvent, "TaxSyncEvent")

function TaxSyncEvent.emptyNew()
    local self = Event.new(TaxSyncEvent_mt)
    return self
end

function TaxSyncEvent.new(taxes)
    local self = TaxSyncEvent.emptyNew()
    self.taxes = taxes or {}
    return self
end

function TaxSyncEvent:writeStream(streamId, connection)
    local count = 0
    for _ in pairs(self.taxes) do count = count + 1 end
    streamWriteUInt8(streamId, count)
    print("TaxesMod: Writing " .. count .. " tax entries")
    for farmId, data in pairs(self.taxes) do
        streamWriteInt32(streamId, farmId)
        streamWriteFloat32(streamId, data.balance)
        streamWriteFloat32(streamId, data.amount)
        print("TaxesMod: Wrote farmId " .. farmId .. ", balance " .. data.balance .. ", amount " .. data.amount)
    end
end

function TaxSyncEvent:readStream(streamId, connection)
    local count = streamReadUInt8(streamId)
    print("TaxesMod: Reading " .. count .. " tax entries")
    for i = 1, count do
        local farmId = streamReadInt32(streamId)
        local balance = streamReadFloat32(streamId)
        local amount = streamReadFloat32(streamId)
        print("TaxesMod: Read farmId " .. farmId .. ", balance " .. balance .. ", amount " .. amount)
        if farmId >= 1 and farmId <= 8 then
            local farm = g_farmManager:getFarmById(farmId)
            if farm then
                farm:changeBalance(amount, MoneyType.OTHER)
                print("TaxesMod: Applied $" .. amount .. " to farm " .. farmId)
            end
            -- Only show GUI if player is initialized and matches farmId
            if g_currentMission.player and farmId == g_currentMission.player.farmId then
                g_gui:showInfoDialog({text = "Quarterly taxes: Balance $" .. balance .. ", Tax paid $" .. -amount})
            end
        else
            print("TaxesMod: Invalid farmId " .. farmId .. " discarded")
        end
    end
end

function TaxSyncEvent:run(connection)
    -- No-op: All work done in readStream for simplicity
end

addModEventListener(TaxesMod)